SystemOrganization addCategory: #SeasideInstaller!

Object subclass: #WAInstaller
	instanceVariableNames: 'zip username password preamble postscript comment version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SeasideInstaller'!

!WAInstaller class methodsFor: 'private' stamp: 'lr 3/15/2007 09:15'!
cleanCategories: aPackage
	aPackage classesAndMetaClasses do: [ :each |
		each organization
			removeEmptyCategories;
			sortCategories ]! !

!WAInstaller class methodsFor: 'configuration' stamp: 'lr 3/15/2007 11:00'!
packages
	"Answer a collection of package names and conditions."

	^ #( ( 'SeasideInstaller' )
		 ( 'DynamicBindings' 'PasteUpMorph confirm: ''Would you like to install the Kom server?''' )
		 ( 'KomServices' 'Smalltalk includesKey: #DynamicBindings' )
		 ( 'KomHttpServer' 'Smalltalk includesKey: #TcpService' )
		 ( 'Seaside2' )
		 ( 'Scriptaculous' 'PasteUpMorph confirm: ''Would you like to install Scriptaculous?''' ) )! !

!WAInstaller class methodsFor: 'initialization' stamp: 'lr 3/15/2007 10:39'!
postload
	"This method will be automatically called after successful loading the distribution."

	self registerRepository: 'http://www.squeaksource.com/Seaside'.
	self packages do: [ :each | self cleanCategories: (PackageInfo named: each first) ]! !

!WAInstaller class methodsFor: 'private' stamp: 'lr 3/15/2007 09:15'!
registerRepository: aString
	| repositoryGroup httpRespository |
	repositoryGroup := Smalltalk classNamed: #MCRepositoryGroup.
	httpRespository := Smalltalk classNamed: #MCHttpRepository.
	(repositoryGroup notNil and: [ httpRespository notNil ]) ifTrue: [
		repositoryGroup default
			addRepository: ((httpRespository
				location: aString
				user: String new
				password: String new)
					creationTemplate: 'MCHttpRepository
	location: ' , aString printString , '
	user: ''''
	password: ''''';
				yourself) ]! !

!WAInstaller methodsFor: 'adding' stamp: 'lr 3/15/2007 09:28'!
addPackage: aPackage
	self addPackage: aPackage workingCopy: (MCWorkingCopy registry at: aPackage)! !

!WAInstaller methodsFor: 'adding' stamp: 'lr 3/15/2007 11:55'!
addPackage: aPackage version: aVersion
	| condition |
	condition := self class packages
		detect: [ :each | aPackage name = each first ].
	condition size > 1
		ifFalse: [ condition := condition copyWith: 'true' ].	
	self zip
		addString: (ByteArray streamContents: [ :stream |
			aVersion fileOutOn: stream ])
		as: aVersion fileName.
	self addPreamble: '(' , condition second , ') ifTrue: ['.
	self addPreamble: '	self fileInMonticelloZipVersionNamed: ' , aVersion fileName printString , ' ].'.
	self addPreamble: '!!'! !

!WAInstaller methodsFor: 'adding' stamp: 'lr 3/15/2007 10:43'!
addPackage: aPackage workingCopy: aWorkingCopy
	aWorkingCopy modified
		ifTrue: [ self error: aPackage name , ' needs to be saved first.' ].
	aWorkingCopy requiredPackages
		do: [ :each | self error: aPackage name , ' has dependencies.' ].
	self
		addPackage: aPackage
		version: (MCRepositoryGroup default
			versionWithInfo: aWorkingCopy ancestors first)! !

!WAInstaller methodsFor: 'adding' stamp: 'lr 3/15/2007 09:32'!
addPostscript: aString
	postscript := self postscript copyWithAll: aString , String cr! !

!WAInstaller methodsFor: 'adding' stamp: 'lr 3/15/2007 09:32'!
addPreamble: aString
	preamble := self preamble copyWithAll: aString , String cr! !

!WAInstaller methodsFor: 'public' stamp: 'lr 3/15/2007 10:45'!
build
	"Build the SAR archive in memory."
	
	zip := preamble := postscript := nil.
	self packages
		do: [ :each | self addPackage: each ]
		displayingProgress: self filename.
	self addPostscript: self class name , ' postload'.
	(self zip addString: self preamble as: 'install/preamble')
		desiredCompressionLevel: 9.
	(self zip addString: self postscript as: 'install/postscript')
		desiredCompressionLevel: 9! !

!WAInstaller methodsFor: 'private' stamp: 'lr 3/15/2007 09:15'!
checkResult: aString
	| result |
	result := #( 'HTTP/1.1 201 ' 'HTTP/1.1 200 ' 'HTTP/1.0 201 ' 'HTTP/1.0 200 ')
		anySatisfy: [ :each | aString beginsWith: each ].
	result ifFalse: [ self error: aString ]! !

!WAInstaller methodsFor: 'accessing' stamp: 'lr 3/15/2007 10:31'!
comment
	^ comment ifNil: [ comment := FillInTheBlank request: 'SqueakMap Comment:' ]! !

!WAInstaller methodsFor: 'public' stamp: 'lr 3/15/2007 09:15'!
dump
	"Dump the SAR archive to the file-system."

	self zip writeToFileNamed: self filename! !

!WAInstaller methodsFor: 'accessing' stamp: 'lr 3/15/2007 10:33'!
filename
	^ 'seaside-' , self version , '.sar'! !

!WAInstaller methodsFor: 'accessing' stamp: 'lr 3/15/2007 10:28'!
packages
	^ self class packages collect: [ :each | MCPackage named: each first ]! !

!WAInstaller methodsFor: 'accessing' stamp: 'lr 3/15/2007 09:26'!
password
	^ password ifNil: [ password := FillInTheBlankMorph request: 'SqueakMap Password:' ]! !

!WAInstaller methodsFor: 'accessing' stamp: 'lr 3/15/2007 09:31'!
postscript
	^ postscript ifNil: [ postscript := String new ]! !

!WAInstaller methodsFor: 'accessing' stamp: 'lr 3/15/2007 09:31'!
preamble
	^ preamble ifNil: [ preamble := String new ]! !

!WAInstaller methodsFor: 'public' stamp: 'lr 3/18/2007 20:18'!
publish
	"Publish the SAR archive as a new release on SqueakMap."

	| result |
	result := HTTPSocket
		httpPost: self squeakMapUrl , 'packagebyname/Seaside/newrelease'
		args: (Array
			with: 'version' -> (Array with: self version)
			with: 'note' -> (Array with: self comment)
			with: 'downloadURL' -> (Array with: self upload))
		user: self username
		passwd: self password.
	(result contents includesSubString: self version)
		ifFalse: [ self error: result contents ]! !

!WAInstaller methodsFor: 'private' stamp: 'lr 3/15/2007 09:15'!
squeakMapUrl
	^ 'http://map1.squeakfoundation.org/sm/'! !

!WAInstaller methodsFor: 'private' stamp: 'lr 3/15/2007 10:47'!
upload
	| result stream |
	result := HTTPSocket
		httpPut: (ByteArray streamContents: [ :s | self zip writeTo: s ])
		to: self squeakMapUrl , 'upload/' , self filename
		user: self username 
		passwd: self password.
	self checkResult: result.
	stream := result readStream.
	stream upToAll: 'http://'.
	^ 'http://' , stream upToEnd! !

!WAInstaller methodsFor: 'accessing' stamp: 'lr 3/15/2007 10:33'!
username
	^ username ifNil: [ username := FillInTheBlank request: 'SqueakMap Username:' initialAnswer: 'lr' ]! !

!WAInstaller methodsFor: 'accessing' stamp: 'lr 3/15/2007 10:33'!
version
	^ version ifNil: [ version := FillInTheBlank request: 'SqueakMap Version:' initialAnswer: '2.8' ]! !

!WAInstaller methodsFor: 'accessing' stamp: 'lr 3/15/2007 09:15'!
zip
	^ zip ifNil: [ zip := ZipArchive new ]! !
